/*
 *  Copyright (c) 2008 Cyrille Berger <cberger@cberger.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation;
 * either version 2, or (at your option) any later version of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _OPENCTL_FUNCTION_P_H_
#define _OPENCTL_FUNCTION_P_H_

#include "Function.h"
#include "Parameter.h"
#include <vector>
#include "GTLCore/ScopedName.h"

namespace llvm {
  class Function;
  class Module;
  class LLVMContext;
}

namespace GTLCore {
  class FunctionCaller;
  class ModuleData;
  /**
   * @internal
   * This class is not part of the public API of OpenCTL.
   * @ingroup GTLCore
   */
  struct Function::Private
  {
    Private() : functionsCaller(0) {}
    ScopedName name;
    const Type* returnType;
    Function::Data* data;
    FunctionCaller* functionsCaller;
    GTLCORE_EXPORT static Function* createExternalFunction(GTLCore::ModuleData* _module, llvm::Module* _llvmModule, llvm::LLVMContext& _context, const GTLCore::Function* _functionToCopy );
    GTLCORE_EXPORT static Function* createExternalFunction(GTLCore::ModuleData* _module, llvm::Module* _llvmModule, llvm::LLVMContext& _context, const GTLCore::String& _name, const GTLCore::String& _symbolName, const GTLCore::Type* retType, const std::vector< GTLCore::Parameter >& arguments);
    enum ExternalFunctionParameters {
      EFP_ONLY_TYPE,
      EFP_TYPE_AND_DEFAULT
    };
    GTLCORE_EXPORT static Function* createExternalFunction(GTLCore::ModuleData* _module, llvm::Module* _llvmModule, llvm::LLVMContext& _context, const GTLCore::String& _name, const GTLCore::String& _symbolName, const GTLCore::Type* retType, ExternalFunctionParameters efpParameter, int _count, ...);
    GTLCORE_EXPORT static Function* createInternalFunction(ModuleData* _module, llvm::LLVMContext& _context, const String& _name, llvm::Function* _function, const Type* retType, ExternalFunctionParameters efpParameter, int _count, ...);
    bool isReturnedAsPointer() const;
  };
  /**
   * @internal
   * This class is not part of the public API of OpenCTL.
   * 
   * This class contains specific structure coming from LLVM,
   * like a pointer to llvm::Function.
   * 
   */
  class Function::Data {
    public:
      Data( const std::vector< Parameter >& _parameters, int minimumParameters);
      void setFunction( llvm::Function* _function );
      void setModule( ModuleData* module );
      /**
       * @param count the number of parameters
       * @return the llvm function corresponding to the number of parameters, or 0 if no
       *         function exist for the given number of parameters
       */
      GTLCORE_EXPORT llvm::Function* function() const;
      ModuleData* module() { return m_module; }
      unsigned int minimumParameters() const { return m_minimumParameters; }
      unsigned int maximumParameters() const { return m_maximumParameters; }
      const std::vector< Parameter >& parameters() { return m_parameters; }
      /**
       * @return the symbol name for the function with the given scoped name and parameters
       */
      GTLCORE_EXPORT static String symbolName( const GTLCore::ScopedName& _functionName, const GTLCore::Type* _returnType, const std::vector< GTLCore::Parameter >& _parameters );
    private:
      std::vector< Parameter > m_parameters;
      llvm::Function* m_function;
      ModuleData* m_module;
      unsigned int m_minimumParameters;
      unsigned int m_maximumParameters;
  };
}

#endif
